import { Actor, Turn } from './actor.js';
import { Dataspace } from './dataspace.js';
import { Relay, spawnRelay } from './relay.js';
import * as net from 'net';

Turn.for(new Actor(), t => {
    const ds = t.ref(new Dataspace());

    function spawnConnection(t: Turn, socket: net.Socket) {
        console.log('connection', socket.remoteAddress, socket.remotePort);
        spawnRelay(t, {
            packetWriter: bs => socket.write(bs),
            setup(t: Turn, r: Relay) {
                socket.on('error', err => t.freshen(t =>
                    ((err as any).code === 'ECONNRESET') ? t.quit() : t.crash(err)));
                socket.on('close', () => t.freshen(t => t.quit()));
                socket.on('end', () => t.freshen(t => t.quit()));
                socket.on('data', data => r.accept(data));
                t.actor.atExit(() => socket.destroy());
            },
            initialRef: ds,
            // debug: true,
        });
    }

    t.spawn(t => {
        const server = net.createServer(socket => t.freshen(t => spawnConnection(t, socket)));
        server.on('error', err => t.freshen(t => t.crash(err)));
        server.listen(5999, '0.0.0.0', 512);
        t.actor.atExit(() => {
            try {
                server.close();
            } catch (e) {
                console.error(e);
            }
        });
    });
});
